unit UFRegionales;

interface
uses
  SysUtils, Variants, Classes, math, UApoyo;

  procedure FR_BordeX    ( Ma : Mat3d; var Mb : Mat3d);
  procedure FR_BordeY    ( Ma : Mat3d; var Mb : Mat3d);
  procedure FR_BordeXY   ( Ma : Mat3d; var Mb : Mat3d);

  procedure FR_BordeConX ( Ma : Mat3d; var Mb : Mat3d;
                           MC : MatC3x3);

  function  Conv3        ( Ma : Mat3d; MC : MatC3x3;
                          x,y,can : integer): single;
                          

  procedure FR_MediaConX ( Ma : Mat3d; var Mb : Mat3d;
                           MC : MatCnxm);

  procedure FR_MedianaS  ( Ma : Mat3d; var Mb : Mat3d);
  
implementation

// Borde X Simple
  procedure FR_BordeX( Ma : Mat3d; var Mb : Mat3d);
  var
    x,y,can : integer;
  begin

    for can := 0 to 2 do if Kanales[can]
      then // aplica el filtro
        for y := _y1 to _y2 - 1 do
          for x := _x1 to _x2 - 2 do
          // ---
            Mb.dat[x][y][can] := Ma.dat[x+1][y][can] -
                                 Ma.dat[x  ][y][can];
          // ---
  end;

// Borde Y Simple
  procedure FR_BordeY( Ma : Mat3d; var Mb : Mat3d);
  var
    x,y,can : integer;
  begin

    for can := 0 to 2 do if Kanales[can]
      then // aplica el filtro
        for y := _y1 to _y2 - 2 do
          for x := _x1 to _x2 - 1 do
          // ---
            Mb.dat[x][y][can] := Ma.dat[x][y+1][can] -
                                 Ma.dat[x][y  ][can];
          // ---
  end;

// Borde X-Y Simple
  procedure FR_BordeXY  ( Ma : Mat3d; var Mb : Mat3d);
  var
    x,y,can : integer;
    dx,dy   : single;
  begin

    for can := 0 to 2 do if Kanales[can]
      then // aplica el filtro
        for y := _y1 to _y2 - 2 do
          for x := _x1 to _x2 - 2 do begin
          // ---
            dx := Ma.dat[x+1][y][can] -
                  Ma.dat[x  ][y][can];
            dy := Ma.dat[x][y+1][can] -
                  Ma.dat[x][y  ][can];
            Mb.dat[x][y][can] := MAX(abs(dx),abs(dy));
          // ---

          end;
  end;

  // Convolucin 3x3
  function  Conv3       ( Ma : Mat3d; MC : MatC3x3;
                          x,y,can : integer):single;
  var
    i,j : integer;
    sum : single;
  begin
    sum := 0;
    for j := -1 to 1 do
      for i := -1 to 1 do
        sum := sum + Ma.dat[x+i][y+j][can]*MC[i][j];

    result := sum;    
  end;

  // Convolucin com MC de 3x3
  procedure FR_BordeConX ( Ma : Mat3d; var Mb : Mat3d;
                           MC : MatC3x3);
var
    x,y,can : integer;
  begin

    for can := 0 to 2 do if Kanales[can]
      then // aplica el filtro
        for y := _y1+1 to _y2 - 2 do
          for x := _x1+1 to _x2 - 2 do
          // ---
            Mb.dat[x][y][can]:= Conv3(Ma,MC, x,y,can)
          // ---
  end;


  // Convolucin con MC de nxm
  procedure FR_MediaConX ( Ma : Mat3d; var Mb : Mat3d;
                           MC : MatCnxm);
  var
    a,b,
    x,y,can : integer;
    ff      : single;

  //  -------  metodo interno  -----
  // Convolucin nxm  medias
  function  Conv3 ( x,y,can : integer):single;
  var
    i,j    : integer;
    sum    : single;
  begin
    sum := 0;
    for j := -b to b do
      for i := -a to a do
        sum := sum + Ma.dat[x+i][y+j][can]*MC.dat[a+i][b+j];

    result := sum;
  end;
  //  -----------------------------

  begin

    a  := MC.nx div 2;
    b  := MC.ny div 2;
    ff := MC.fac;

    for can := 0 to 2 do if Kanales[can]
      then // aplica el filtro
        for y := _y1+b to _y2 - b-1 do
          for x := _x1+a to _x2 - a-1 do
          // ---
            Mb.dat[x][y][can]:= Conv3(x,y,can)/ff;
          // ---
  end;

  // Mediana simple 3x3
  procedure FR_MedianaS  ( Ma : Mat3d; var Mb : Mat3d);
  var
    a,b,
    med,lon,
    x,y,can   : integer;
    vv        : array of single;

  procedure recoge (xx,yy : integer);
  var
    i,j,kk  : integer;
  begin
    kk := 0;
    for j := -b to b do
      for i := -a to a do begin
        vv[kk] := Ma.dat[xx+i][yy+j][can];
        inc(kk);
      end;
  end;

  function mediana(xx,yy : integer):single;
  var
    i,j,i1  : integer;
    bb      : boolean;
    tt      : single;
  begin
    recoge(xx,yy);

    for j := 0 to lon - 2 do begin
      bb := false;
      for i := 0 to lon - 2 - j do begin
         i1 := i+1;
         if (vv[i]>vv[i1]) then begin
           tt     := vv[ i];
           vv[i]  := vv[i1];
           vv[i1] := tt;
           bb := true;
         end;
      end;
      if not bb then break;
    end;

    result := vv[med];
  end;

  { mtodo principal para evaluacin de la mediana}
  begin
    a  := 3 div 2;    // 3: tamao horizontal de la mscara
    b  := 3 div 2;    // 3: tamao vertical de la mscara

    lon := 3*3;
    med := lon div 2;
    SetLength(vv,lon);

    for can := 0 to 2 do if Kanales[can]
      then // aplica el filtro
        for y := _y1+b to _y2 - b-1 do
          for x := _x1+a to _x2 - a-1 do
          // ---
            Mb.dat[x][y][can]:= mediana(x,y);
          // ---

  end;


end.
